/* prog to read and change rppm's - test only */

/* Copyright Peter Murray-Rust, Glaxo, 1994 
   this software may be freely distributed and edited so long as this 
   copyright is included.  The software may not be sold.
   for details see the copyright supplied with the Mapmarker software 
*/
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "imagelib.h"

int floodFill4(int x, int y, int nx, int ny, USC* array, USC* colour,
    USC* edgeArray, USC* edgeColour);
int getColour(int x, int y, int nx, int ny, USC* array, USC* colour);
int putColour(int x, int y, int nx, int ny, USC* array, USC* colour);
int fillPoint4(int x, int y, int nx, int ny, USC* array, USC* fillColour,
    USC* edgeArray, USC* edgeColour);
int fillQueue4(int x, int y);
int fillUnQueue4(int* x, int* y);
int storeXYPoint(int x, int y);
int joinDots(int nx, int ny, USC* array);
int drawChain(USC* edgeColour, int width, int nx, int ny, USC* array);
int areNeighbours(int n1, int n2);
int pruneSpikes();
int deTriangulate();

#define MAXQUEUE 5000
#define MAXPTS 10000
#define MAXEDGES 100
#define MAXCHAIN 20
#define MAXLINELEN 5000

static USC oldColour[3];
static USC fillColour[3];
static int queuex[MAXQUEUE], queuey[MAXQUEUE];
static int firstInQueue, lastInQueue;
static int nedges;
static int nedgepts;
static int edgepts[2][MAXPTS];
static int edges[2][MAXEDGES];
static int neighbours[8][MAXPTS];
static int nneighbours[MAXPTS];
static int chain[MAXCHAIN][MAXPTS];
static int nchains;
static int ninchain[MAXCHAIN];
static int validChain[MAXCHAIN];
static debug = 0;

int main(int argc, char** argv)
{
    FILE* inf;
    FILE* outf;
    int nx, ny, nlev, i, j, x0, y0;
    USC* array;
    char infile[80], outfile[80];
    char *intype, *outtype;
    char* junk;

/* read input and output files */
    
    if (argc != 3)
    {
        showUsage();
        return 0;
    }
    strcpy(infile,argv[1]);
    strcpy(outfile,argv[2]);
    inf = fopen(infile, "r");
    if (inf == 0) 
    {
        printf("Cannot open file: %s\n", infile);
        return 1;
    }
    intype = getSuffix(infile);
    outf = fopen(outfile, "w");
    if (outf == 0) 
    {
        printf("Cannot open outfile: %s\n", outfile);
        return 1;
    }
    outtype = getSuffix(outfile);

    readImage1(intype,inf, &nx, &ny, &nlev);
    array = (USC*) malloc(3*nx*ny);
    readImage2(intype,inf, nx, ny, array);

    fclose(inf);

    editImage(nx, ny, array);

    if (strncmp(outtype, "edges", 5) == 0)
    {
        writeEdges(outf);
    }
    else
    {
        writeImage(outtype,outf, nx, ny, nlev, array);
        if (debug)
        {
            printf("DEBUG: WRITING: %s %d %d\n", outtype, nx, ny);
        }
    }
    fclose(outf);

    free (array);

    return 0;
}

int editImage(int nx, int ny, USC* array)
{

    int* xyz;
    int rx, gx, bx;
    char* nexttok;
    char line[MAXLINELEN], rawline[MAXLINELEN], junk[MAXLINELEN];
    USC red, green, blue, *tempArray;
    USC newColour[3], edgeColour[3];
    int xval, yval, xcurr, ycurr, xx, x, y, biggestChain;
    int offset, firstnonspace;
    int xc, yc, l, width, x0, y0, x1, y1, radius, i, val;
    char text[80];

/* defaults */
    red = 255;
    green = 255;
    blue = 255;
    width = 1;
    nedges = 16;

    while (1)
    {
        firstnonspace = -1;
        if (fgets(rawline, MAXLINELEN, stdin) == 0)
        {
            break;
        }
        l = strlen(rawline);
        for (i = 0; i <= l; i++)
        {
/* uppercase and substitute commas or tabs by spaces */
            line[i] = toupper(rawline[i]);
            if (line[i] == ',') line[i] = ' ';
            if (line[i] == '\t') line[i] = ' ';
            if (firstnonspace == -1 && line[i] != ' ' && i < l)
            {
                firstnonspace = i;
            }
        }
/* left justify */
        strcpy(junk, line+firstnonspace);
        strcpy(line, junk);
        strcpy(junk, rawline+firstnonspace);
        strcpy(rawline, junk);
        if (debug)
        {
            printf("DEBUG: INPUT:  %s\n", rawline);
        }

/* skip blank lines */
        if (strlen(rawline) == 0 || rawline[0] == '\n')
        {
        }
/* jump leading '#' as comment */
        else if (line[0] == '#')
        {
            if (debug)
            {
                printf("DEBUG: COMMENT:  %s\n", line);
            }
        }
        else if (strncmp(line, "RGB", 3) == 0)
        {
            sscanf(line, "%s %d %d %d", junk, &rx, &gx, &bx);
            red = (USC) rx;
            green = (USC) gx;
            blue = (USC) bx;
            if (debug)
            {
                printf("DEBUG: RGB:  %d %d %d\n", red, green, blue);
            }
        }
        else if (strncmp(line, "GRID", 4) == 0)
        {
            if (debug)
            {
                printf("DEBUG: GRID: \n");
            }
            drawGrid(nx, ny, array);
        }
        else if (strncmp(line, "WIDTH", 5) == 0)
        {
            sscanf(line, "%s %d", junk, &width);
            if (debug)
            {
                printf("DEBUG: WIDTH: %d \n", width);
            }
        }
        else if (strncmp(line, "TEXT", 4) == 0)
        {
            sscanf(rawline, "%s %d %d %s", junk, &x0, &y0, text);
/* jump the first three fields */
            nexttok = strtok(line, " ");
            nexttok = strtok(0, " ");
            nexttok = strtok(0, " ");
            nexttok = strtok(0, " ");
            offset = nexttok-line;
            plotText(x0, y0, red, green, blue, rawline+offset, nx, ny, array);
            if (debug)
            {
                printf("DEBUG: TEXT: %d %d %s \n", x0, y0, text);
            }
        }
        else if (strncmp(line, "LINE", 4) == 0)
        {
            sscanf(line, "%s %d %d %d %d", junk, &x0, &y0, &x1, &y1);
            plotLine(x0, y0, x1, y1, red, green, blue, width, nx,ny,array);
        }
        else if (strncmp(line, "CIRC", 4) == 0)
        {
            sscanf(line, "%s %d %d %d", junk, &x0, &y0, &radius);
            plotCircle(x0, y0, radius, red, green, blue, width, nx,ny,array);
        }
        else if (strncmp(line, "RECT", 4) == 0)
        {
            sscanf(line, "%s %d %d %d %d", junk, &x0, &y0, &x1, &y1);
            plotLine(x0, y0, x0, y1, red, green, blue, width, nx, ny, array);
            plotLine(x0, y1, x1, y1, red, green, blue, width, nx, ny, array);
            plotLine(x1, y1, x1, y0, red, green, blue, width, nx, ny, array);
            plotLine(x1, y0, x0, y0, red, green, blue, width, nx, ny, array);
        }
        else if (strncmp(line, "POLY", 4) == 0)
        {
            xcurr = -999;
            ycurr = -999;
            xx = 0;
            nexttok = strtok(line, " ");
            while ( (nexttok = strtok(0, " ")) != (char*) 0)
            {
                sscanf(nexttok, "%d", &val);
                if (xx == 0) 
                {
                    xx = 1;
                    xval = val;
                }
                else
                {
                    xx = 0;
                    yval = val;
                    if (xcurr > 0)
                    {
                        plotLine(xcurr, ycurr, xval, yval, red, green, blue, 
                            width, nx, ny, array);
                    }
                    xcurr = xval;
                    ycurr = yval;
                }
            }
        }
        else if (strncmp(line, "DEBUG", 5) == 0)
        {
            debug = 1-debug;
            if (debug == 0) 
            {
                printf("Debug is now OFF\n");
            }
            else
            {
                printf("Debug is now ON\n");
            }
        }
        else if (strncmp(line, "FLOOD", 5) == 0)
        {
            sscanf(line, "%s %d %d", junk, &x, &y);
            newColour[0] = red;
            newColour[1] = green;
            newColour[2] = blue;
            if (debug)
            {
                printf("FloodFill: %d %d %d %d %d\n", red, green, blue, x, y);
            }
            floodFill4(x, y, nx, ny, array, newColour, 0, 0);
        }
        else if (strncmp(line, "NEDGES", 4) == 0)
        {
            sscanf(line, "%s %d", junk, &x);
            if (x >= 4 && x <= 50)
            {
                nedges = x;
            }
        }
        else if (strncmp(line, "EDGE", 4) == 0)
        {
            sscanf(line, "%s %d %d", junk, &x, &y);
            edgeColour[0] = red;
            edgeColour[1] = green;
            edgeColour[2] = blue;
            tempArray = (USC*) malloc((long)3*(nx)*(ny));
            memcpy (tempArray, array, (long)3*(nx)*(ny));
            nedgepts = 0;
            floodFill4(x, y, nx, ny, tempArray, newColour, array, edgeColour);
            free (tempArray);
            biggestChain = joinDots(nx, ny, array);
            if (biggestChain != -1)
            {
                drawChain(edgeColour, width, nx, ny, array);
            }
        }
        else if (line == "" || (strncmp(line, "QUIT", 4) == 0))
        {
            break;
        }
        else
        {
            printf("Bad input line: %d %s\n", strlen(line), line);
        }
    }
}

/* fills the point at array(x,y) with fillColour if it is OldColour:
    if it is fillColour already do nothing
    if it is another colour assume it is the boundary 
        if edgeArray is NOT null, fill it with edgeColour
*/
int fillPoint4(int x, int y, int nx, int ny, USC* array, USC* fillColour,
    USC* edgeArray, USC* edgeColour)
{
    USC newColour[3];
    getColour(x, y, nx, ny, array, newColour);
    if (newColour[0] == oldColour[0] &&
        newColour[1] == oldColour[1] &&
        newColour[2] == oldColour[2])
    {
        putColour(x, y, nx, ny, array, fillColour);
        fillQueue4( x, y);
    }
    else if (newColour[0] == fillColour[0] &&
        newColour[1] == fillColour[1] &&
        newColour[2] == fillColour[2])
    {
        /* do nothing */
    }
    else
    {
        if (edgeArray != 0)
        {
            getColour(x, y, nx, ny, edgeArray, newColour);
            if (newColour[0] == edgeColour[0] &&
                newColour[1] == edgeColour[1] &&
                newColour[2] == edgeColour[2])
            {
                 /* don't store point twice */
            }
            else
            {
                putColour(x, y, nx, ny, edgeArray, edgeColour);
                storeXYPoint(x, y);
            }
        }
    }
    return 0;
}

int fillQueue4(int x, int y)
{
    lastInQueue = (lastInQueue+1)%MAXQUEUE;
    queuex[lastInQueue] = x;
    queuey[lastInQueue] = y;
    return 0;
}

int fillUnQueue4(int* x, int* y)
{
    *x = queuex[firstInQueue];
    *y = queuey[firstInQueue];
    firstInQueue = (firstInQueue+1)%MAXQUEUE;
    return 0;
}


/* array is the array whose *area* is to be filled and if edgeArray 
    is NOT 0, it is the one to have its edges filled (array is then usually
    a work area.  If edgeArray is 0, then edgeColour is not used.
*/
int floodFill4(int x, int y, int nx, int ny, USC* array, USC* fillColour,
    USC* edgeArray, USC* edgeColour)
{
    firstInQueue = 1;
    lastInQueue = 0;
    getColour(x, y, nx, ny, array, oldColour);
    if (debug) 
    {
        printf("floodFill4: %d %d %d\n", \
                oldColour[0], oldColour[1], oldColour[2]);
    }
    if (edgeArray != 0)
    {
/* temporary fill colour - just tweak the old colour very little */
        fillColour[0] = oldColour[0];
        fillColour[1] = oldColour[1];
        fillColour[2] = oldColour[2];
        if (fillColour[0] > 0) 
        {
            fillColour[0]--; 
        }
        else
        {
            fillColour[0]++;
        }
    }
    fillPoint4(x, y, nx, ny, array, fillColour, edgeArray, edgeColour);
    while (1)
    {
        fillUnQueue4(&x, &y);
        fillPoint4(x, y+1, nx, ny, array, fillColour, edgeArray, edgeColour);
        fillPoint4(x, y-1, nx, ny, array, fillColour, edgeArray, edgeColour);
        fillPoint4(x+1, y, nx, ny, array, fillColour, edgeArray, edgeColour);
        fillPoint4(x-1, y, nx, ny, array, fillColour, edgeArray, edgeColour);
        if (firstInQueue == (lastInQueue+1)%MAXQUEUE) break;
    }
    return 0;
}

int getColour(int x, int y, int nx, int ny, USC* array, USC* colour)
{
    int k;
    if (x < 0 || x >= nx || y < 0 || y >= ny) return 1;
    for (k = 0; k < 3; k++)
    {
        colour[k] = array[k+3*x+3*nx*y];
    }
    return 0;
}

int putColour(int x, int y, int nx, int ny, USC* array, USC* colour)
{
    int k;
    if (x < 0 || x >= nx || y < 0 || y >= ny) return 1;
    for (k = 0; k < 3; k++)
    {
        array[k+3*x+3*nx*y] = colour[k];
    }
    return 0;
}

int storeXYPoint(int x, int y)
{
    edgepts[0][nedgepts] = x;
    edgepts[1][nedgepts] = y;
    if (nedgepts >= MAXPTS-1) 
    {
        printf("ERROR: TOO MANY POINTS ON EDGE, SORRY\n");
        return 0;
    }
    ++nedgepts;
    return 0;
}

int joinDots(int nx, int ny, USC* array)
{
    int i, j, k, xi, yi, xj, yj, ii, start, current, next, change;
    int biggest;
    for (i = 0; i < MAXPTS; i ++)
    {
        nneighbours[i] = 0;
    }

    for (i = 0; i < nedgepts-1; i++)
    {
        xi = edgepts[0][i];
        yi = edgepts[1][i];
        for (j = i+1; j < nedgepts; j++)
        {
            xj = edgepts[0][j];
            yj = edgepts[1][j];
            if ((abs(xi - xj) <= 1) && (abs(yi - yj) <= 1))
            {
                neighbours[nneighbours[i]][i] = j;
                neighbours[nneighbours[j]][j] = i;
                nneighbours[i]++;
                nneighbours[j]++;
            }
        }
    }

    while (1)
    {
        change = pruneSpikes();
        change += deTriangulate();
        change += deleteCongested();
        change += breakSquare();
        if (change == 0) break;
    }

    makeChain();
    biggest = getLargestChain();
    if (biggest == -1)
    {
        printf("ERROR: Region too complex\n");
    }
    return biggest;

}

int getBox(int ichain, int* xmin, int* ymin, int* xmax, int* ymax)
{
    int j, x, y, ipoint;
    *xmin= 9999; *ymin = 9999; *xmax = -9999; *ymax = -9999;
    for (ipoint = 0; ipoint < ninchain[ichain]; ipoint++)
    {
        j = chain[ichain][ipoint];
        x = edgepts[0][j];
        y = edgepts[1][j];
        if (x < *xmin)
        {
            *xmin = x;
        }
        if (x > *xmax)
        {
            *xmax = x;
        }
        if (y < *ymin)
        {
            *ymin = y;
        }
        if (y > *ymax)
        {
            *ymax = y;
        }
    }
    return 0;
}

int getLargestChain()
{
    int ichain, biggest;
    int xb0, xb1, yb0, yb1, x0, y0, x1, y1;
    biggest = -1;
    xb0 = 9999; yb0 = 9999; xb1 = -9999; yb1 = -9999;
    for (ichain = 0; ichain < nchains; ichain++)
    {
        if (validChain[ichain])
        {
            getBox(ichain, &x0, &y0, &x1, &y1);
            if (x0 > xb0 && y0 > yb0 && x1 < xb1 && y1 < yb1)
            {
                validChain[ichain] = 0;
            }
            else if (x0 < xb0 && y0 < yb0 && x1 > xb1 && y1 > yb1)
            {
                validChain[biggest] = 0;
                biggest = ichain;
                xb0 = x0; yb0 = y0; xb1 = x1; yb1 = y1;
            }
        }
    }
    return biggest;
}

int reDraw(int nx, int ny, USC* array)
{
    int ii, i;
    USC colour[3];
    colour[0] = 0;
    colour[1] = 255;
    colour[2] = 255;
    
    for (i = 0; i < nedgepts; i++)
    {
        if (nneighbours[i] > 1)
        {
            putColour(edgepts[0][i], edgepts[1][i], nx, ny, array, colour);
        }
    }
    return 0;
}

int deleteCongested()
{
    int i, neighbour, change, anyChange, j;
    anyChange = 0;
    while (1)
    {
        change = 0;
        for (i = 0; i < nedgepts-1; i++)
        {
            if (nneighbours[i] >= 5)
            {
                for (j = 0; j < nneighbours[i]; j++)
                {
                    neighbour = neighbours[j][i];
                    deleteNeighbour(i, neighbour);
                    deleteNeighbour(neighbour, i);
                }
                change = 1;
                anyChange = 1;
            }
        }
        if (change == 0) break;
    }
    return anyChange;
}

int breakSquare()
{
    int i, neighbour, change, anyChange, j, n1, n2, n3;
    anyChange = 0;
    while (1)
    {
        change = 0;
        for (i = 0; i < nedgepts-1; i++)
        {
            if (nneighbours[i] == 3)
            {
                n1 = neighbours[0][i];
                n2 = neighbours[1][i];
                n3 = neighbours[2][i];
                if (areNeighbours(n1, n2) &&
                    areNeighbours(n1, n3) &&
                    areNeighbours(n2, n3))
                {
                    deleteNeighbour(i, n1);
                    deleteNeighbour(i, n2);
                    deleteNeighbour(i, n3);
                    deleteNeighbour(n1, i);
                    deleteNeighbour(n2, i);
                    deleteNeighbour(n3, i);
                    change = 1;
                    anyChange = 1;
                 }
           }
        }
        if (change == 0) break;
    }
    return anyChange;
}

int pruneSpikes()
{
    int i, neighbour, change, anyChange;
    anyChange = 0;
    while (1)
    {
        change = 0;
        for (i = 0; i < nedgepts-1; i++)
        {
            if (nneighbours[i] == 1)
            {
                neighbour = neighbours[0][i];
                deleteNeighbour(i, neighbour);
                deleteNeighbour(neighbour, i);
                change = 1;
                anyChange = 1;
            }
        }
        if (change == 0) break;
    }
    return anyChange;
}
    
    
int makeChain()
{
    int used[MAXPTS];
    int i, j, current, start, next, total, isthmus;

    nchains = 0;
    for (j = 0; j < MAXCHAIN; j++)
    {
        ninchain[j] = 0;
        validChain[j] = 0;
    }
    for (i = 0; i < nedgepts; i++)
    {
        for (j = 0; j < MAXCHAIN; j++)
        {
            chain[j][i] = 0;
        }
        used[i] = -1;
    }
    while (1)
    {
        ninchain[nchains] = 0;
        start = getNextUnused(used);
/* used up all points? */
        if (start == -1) 
        {
            break;
        }
        current = start;
        total = 0;
        isthmus = -1;
        while (1)
        {
            if (nneighbours[current] >= 3)
            {
                isthmus = current;
                used[current] = 1;
                ninchain[nchains] = 0;
                break;
            }
            else if (nneighbours[current] == 0)
            {
                used[current] = 1;
                ninchain[nchains] = 0;
                break;
            }
            else if (nneighbours[current] == 1)
            {
                used[current] = 1;
                ninchain[nchains] = 0;
                break;
            }
            next = getNextUnusedNeighbour(current, used);
            if (next == -1)
            {
                if (!areNeighbours(current, start))
                {
                    break;
                }
            }
            chain[nchains][ninchain[nchains]] = current;
            used[current] = ninchain[nchains];
            ninchain[nchains]++;
            current = next;
            total++;
            if (next == -1) break;
        }
        if (ninchain[nchains] > 0) 
        {
            validChain[nchains] = 1;
            nchains++;
            if (nchains >= MAXCHAIN)
            {
                printf("ERROR: Too many chains in region\n");
                return 1;
            }
        }
    }
    return 0;
}
 
int getNextUnused(int* used)
{
    int i;
    for (i = 0; i < nedgepts; i++)
    {
        if (used[i] == -1)
        {
            return i;
        }
    }
    return -1;
}    

int getNextUnusedNeighbour(int current, int* used)
{
    int i, next;
    for (i = 0; i < nneighbours[current]; i++)
    {
        next = neighbours[i][current];
        if (used[next] == -1) 
        {
            return next;
        }
    }
    return -1;
}

int drawChain(USC* colour, int width, int nx, int ny, USC* array)
{
    int i, j, k, ii, x0, y0, x1, y1, ninch, ichain;
    USC red, green, blue;

    red = colour[0];
    green = colour[1];
    blue = colour[2];
    for (ichain = 0; ichain < nchains; ichain++)
    {
        if (validChain[ichain])
        {
            ninch = ninchain[ichain];
            for (i = 0; i < nedges; i++)
            {
                ii = (i+1)%nedges;
                j = chain[ichain][(ninch*i)/nedges];
                k = chain[ichain][(ninch*ii)/nedges];
                x0 = edgepts[0][j];
                y0 = edgepts[1][j];
                x1 = edgepts[0][k];
                y1 = edgepts[1][k];
                plotLine(x0, y0, x1, y1,red, green, blue, width, nx, ny, array);
            }
        }
    }
    return 0;
}

int deTriangulate()
{
    int i, j, n1, n2, neighbour, change, anyChange;
    anyChange = 0;
    while (1)
    {
        change = 0;
        for (i = 0; i < nedgepts-1; i++)
        {
            if (nneighbours[i] == 2)
            {
                n1 = neighbours[0][i];
                n2 = neighbours[1][i];
                if (areNeighbours(n1, n2))
                {
                    deleteNeighbour(i, n1);
                    deleteNeighbour(i, n2);
                    deleteNeighbour(n1, i);
                    deleteNeighbour(n2, i);
                    anyChange = 1;
                    change = 1;
                }
            }
        }
        if (change == 0) break;
    }
    return anyChange;
}

int areNeighbours(int n1, int n2)
{
    int i;
    if (n1 >= 0 && n2 >= 0 && n1 < nedgepts && n2 < nedgepts)
    {
        for (i = 0; i < nneighbours[n1]; i++)
        if (neighbours[i][n1] == n2) 
        {
            return 1;
        }
    }
    return 0;
}

int deleteNeighbour(int i, int j)
{
    int k, kk;
    kk = 0;
    for (k = 0; k < nneighbours[i]; k++)
    {
        if (neighbours[k][i] != j)
        {
            neighbours[kk][i] = neighbours[k][i];
            kk++;
        }
    }
    nneighbours[i]--;
    return 0;
}
       
int drawGrid(int nx, int ny, USC* array)
{
    char text[80];
    USC red, green, blue;
    int i, j, k;
    red = 100; green = 100; blue = 100;
    for (i = 0; i < nx; i += 25)
    {
        for (j = 0; j < ny; j++)
        {
            for (k = 0; k < 3; k++)
            {
                array[k + 3*i + 3*nx*j] = 255 - array[k + 3*i + 3*nx*j];
            }
        }
    }
    for (i = 1; i < nx; i += 100)
    {
        sprintf(text,"%d", i-1);
        plotText(i, 10, red, green, blue, text, nx,ny,array);
        for (j = 0; j < ny; j++)
        {
            for (k = 0; k < 3; k++)
            {
                array[k + 3*i + 3*nx*j] = 255 - array[k + 3*i + 3*nx*j];
            }
        }
    }
    for (j = 0; j < ny; j += 25)
    {
        for (i = 0; i < nx; i++)
        {
            for (k = 0; k < 3; k++)
            {
                array[k + 3*i + 3*nx*j] = 255 - array[k + 3*i + 3*nx*j];
            }
        }
    }
    for (j = 1; j < ny; j += 100)
    {
        sprintf(text,"%d", j-1);
        plotText(0, j, red, green, blue, text, nx,ny,array);
        for (i = 0; i < nx; i++)
        {
            for (k = 0; k < 3; k++)
            {
                array[k + 3*i + 3*nx*j] = 255 - array[k + 3*i + 3*nx*j];
            }
        }
    }
    return 0;
}

int showUsage()
{
    printf(" Usage\n\
    editimage  <inputfile> <outputfile>\n\
\n\
    suffixes:\n\
        rgb\n\
        rppm\n\
        edges (output only)\n\
\n");
    printf("\
    control (all optional except QUIT) (on stdin)\n\
        set next operation colour\n\
            RGB <red> <blue> <green>  (0-255)\n\
        set next operation width\n\
            WIDTH <w>\n\
        flood fill area containing point at x,y\n\
            FLOOD <x> <y>\n\
        find edges of region containing point x,y and colour\n\
        (if output is to *.edges, write polygonal edges (POLY))\n\
            EDGE <x> <y>\n");
    printf("\
        draw circle\n\
            CIRCLE <x> <y> <rad>\n\
        draw rectangle\n\
            RECT <x1> <y1> <x2> y2>\n\
        draw line \n\
            LINE <x1>,<y1> <x2>,<y2>\n\
        draw polygon\n\
            POLY <x1>,<y1> <x2>,<y2> ...\n\
        draw grid every 25 points\n\
            GRID\n\
        number of edges to determine for output\n\
            NEDGES <n>\n\
        turn on some crude debug printing\n\
            DEBUG\n\
        quit\n\
            QUIT\n\
\n");
}

int writeEdges(FILE* outf)
{
    int i, j, ichain, x0, y0;
    fprintf(outf, "POLY ");
    for (ichain = 0; ichain < nchains; ichain++)
    {
        for (i = 0; i < nedges; i++)
        {
            j = chain[ichain][(ninchain[ichain]*i)/nedges];
            x0 = edgepts[0][j];
            y0 = edgepts[1][j];
            fprintf(outf, "%d,%d ", x0, y0);
        }
    }
    return 0;
}
        
int plotText(int x0, int y0, USC red, USC green, USC blue, char* text, 
    int nx, int ny, USC* array)
{
    int l, i;
    l = strlen(text);
    for (i = 0; i < l; i++)
    {
        plotChar(x0+8*i, y0, red, green, blue, text[i], nx,ny,array);
    }
    return 0;
}
