/* library to read , edit and write various image formats */

/* Copyright Peter Murray-Rust, Glaxo, 1994 
   this software may be freely distributed and edited so long as this 
   copyright is included.  The software may not be sold.
   for details see the copyright supplied with the Mapmarker software 
*/


#include <stdio.h>
#include <string.h>
#include <math.h>

#include "imagelib.h"
#include "font.h"

int readRPPM1(FILE* inf, int* nx, int* ny, int* nlev);
int readRPPM2(FILE* inf, int nx, int ny, USC* array);
int writeRPPM(FILE* outf, int nx, int ny, int nlev, USC* array);
int readRGB1(FILE* inf, int* nx, int* ny, int* nlev);
int readRGB2(FILE* inf, int nx, int ny, USC* array);
int writeRGB(FILE* outf, int nx, int ny, int lev, USC* array);
int readXBM1(FILE* inf, int* nx, int* ny, int* nlev);
int readXBM2(FILE* inf, int nx, int ny, USC* array);
int readXbmSize(char* line, int* npix);

int readImage1(char* type, FILE* inf, int* nx, int* ny, int* nlev)
{
    if(strncmp("rppm", type, 4) == 0)
    {
        readRPPM1(inf, nx, ny, nlev);
    }
    else if(strncmp("rgb", type, 3) == 0)
    {
        readRGB1(inf, nx, ny, nlev);
    }
    else if(strncmp("xbm", type, 3) == 0)
    {
        readXBM1(inf, nx, ny, nlev);
    }
    else
    {
        printf("Unknown file type: %s\n", type);
        return 1;
    }
    return 0;
}

int readImage2(char* type, FILE* inf, int nx, int ny, USC* array)
{
    if(strncmp("rppm", type, 4) == 0)
    {
        readRPPM2(inf, nx, ny, array);
    }
    else if(strncmp("rgb", type, 3) == 0)
    {
        readRGB2(inf, nx, ny, array);
    }
    else if(strncmp("xbm", type, 3) == 0)
    {
        readXBM2(inf, nx, ny, array);
    }
    else
    {
        printf("Unknown file type: %s\n", type);
        return 1;
    }
    return 0;
}

int writeImage(char* type, FILE* outf, int nx, int ny, int nlev, USC* array)
{
    if(strncmp("rppm", type, 4) == 0)
    {
        writeRPPM(outf, nx, ny, nlev, array);
    }
    else if(strncmp("rgb", type, 3) == 0)
    {
        writeRGB(outf, nx, ny, nlev, array);
    }
    else if(strncmp("xbm", type, 3) == 0)
    {
        writeXBM(outf, nx, ny, nlev, array);
    }
    else
    {
        printf("Unknown file type: %s\n", type);
        return 1;
    }
    return 0;
}


int readRPPM1(FILE* inf, int* nx, int* ny, int* nlev)
{
    char first[80];
    fgets(first, 80, inf);
    while (1)
    {
        fgets(first, 80, inf);
        if (first[0] != '#') break;
    }
    sscanf(first, "%d %d\n", nx, ny);
    fscanf(inf, "%d\n", nlev);
    return 0;
}

int readRPPM2(FILE* inf, int nx, int ny, USC* array)
{
    USC cc;
    int i, j, k, c;
    for (j = 0; j < ny; j++) 
    {
        for (i = 0; i < nx; i++)
        {
            for (k = 0; k < 3; k++) 
            {
                c = getc(inf);
                if (c < 0) break;
                cc = (USC) c;
                array[k+3*i+3*(nx)*j] = cc;
            }
        }
    }
    return 0;
}

int writeRPPM(FILE* outf, int nx, int ny, int nlev, USC* array)
{
    USC cc;
    int i, j, k;
    fprintf(outf, "P6\n");
    fprintf(outf, "%d %d\n", nx, ny);
    fprintf(outf, "%d\n", nlev);
    for (j = 0; j < ny; j++)
    {
        for (i = 0; i < nx; i++)
        {
            for (k = 0; k < 3; k++)
            {
                cc = array[k+3*i+3*nx*j];
                fprintf(outf, "%c",cc);
            }
        }
    }
    return 0;
}

int plotPixel(int x, int y, USC r, USC g, USC b, int nx, int ny, USC* array)
{
    if (x < 0 || x >= nx || y < 0 || y >= ny) return 1;
    array[0+3*x+3*nx*y] = r;
    array[1+3*x+3*nx*y] = g;
    array[2+3*x+3*nx*y] = b;
    return 0;
}

int plotCircle(int xc, int yc, int radius, USC r, USC g, USC b,
 int width, int nx, int ny, USC* array)
{
    int x, y, d, rad;
    int r1, r2;
    r1 = radius - (width-1)/2;
    r2 = radius + width/2;
    if (r1 < 0) r1 = 1;
    for (rad = r1; rad <= r2; rad++)
    {
        x = 0;
        y = rad;
        d = 3 - 2*rad;
        while (x < y) 
        {
            circlePoints(xc, yc, x, y, r, g, b, nx, ny, array);
            if (d < 0)
            {
                d = d + 4*x +6;
            }
            else
            {
                d = d + 4*(x-y) + 10;
                y = y - 1;
            }
            x = x + 1;
        }
        if (x == y)
        {
            circlePoints(xc, yc, x, y, r, g, b, nx, ny, array);
        }
    }
    return 0;
}
    
int circlePoints(int xc, int yc, int x, int y, USC r, USC g, USC b,
 int nx, int ny, USC* array)
{
    plotPixel(xc + x, yc + y, r, g, b, nx, ny, array);
    plotPixel(xc - x, yc + y, r, g, b, nx, ny, array);
    plotPixel(xc + x, yc - y, r, g, b, nx, ny, array);
    plotPixel(xc - x, yc - y, r, g, b, nx, ny, array);
    plotPixel(xc + y, yc + x, r, g, b, nx, ny, array);
    plotPixel(xc - y, yc + x, r, g, b, nx, ny, array);
    plotPixel(xc + y, yc - x, r, g, b, nx, ny, array);
    plotPixel(xc - y, yc - x, r, g, b, nx, ny, array);
    return 0;
}

int getOctant(int x1, int y1, int x2, int y2)
{
/* assumes that x1 < x2 */
    int dx, dy;
    dx = x2 - x1;
    dy = y2 - y1;
    
    if (dy > 0)
    {
       if (dx > dy) 
       {
           return 1;
       }
       return 2;
    }
    dy = -dy;
    if (dx > dy) 
    {
        return 8;
    }
    return 7;
}

int plotChar(int x1, int y1, USC r, USC g, USC b, int charx,
 int nx, int ny, USC* array)
{
    int x, y;

    charx -= 32;
    if (charx < 0) return 0;
    for (x = 0; x < 8; x++)
    {
        for (y = 0; y < 16; y++)
        {
            if (font[charx][y][x] == 1)
            {
                plotPixel(x1 + x, y1 + y, r, g, b, nx, ny, array);
            }
        }
    }
    return 0;
}

int plotLine(int x1, int y1, int x2, int y2, USC r, USC g, USC b, int width, int
 nx, int ny, USC* array)
{
    int dx, dy, dx1, dy1, l, l1, l2;
    double d, cost, sint;
    dx = x2 - x1;
    dy = y2 - y1;
    d = sqrt((double)(dx*dx + dy*dy));
    sint = (double)dy/d;
    cost = (double)dx/d;
    l1 = -((width -1)/2);
    l2 = (width)/2;
/* plot 2 lines to make sure that we don't get gaps */
    for (l = l1; l <= l2; l++)
    {
        dx1 = (double)l*sint;
        dy1 = -(double)l*cost;
        plotLine1(x1+dx1, y1+dy1, x2+dx1, y2+dy1, r, g, b, nx, ny, array);
        dx1 = ((double)l+0.5)*sint;
        dy1 = -((double)l+0.5)*cost;
        plotLine1(x1+dx1, y1+dy1, x2+dx1, y2+dy1, r, g, b, nx, ny, array);
    }
    return 0;
}

int plotLine1(int x1, int y1, int x2, int y2, USC r, USC g, USC b,
 int nx, int ny, USC* array)
{
    int oct, temp;

/* make sure x2 > x1 */
    if (x1 > x2)
    {
        temp = x1;
        x1 = x2;
        x2 = temp;
        temp = y1;
        y1 = y2;
        y2 = temp;
    }
    oct = getOctant(x1, y1, x2, y2);
    
    linePoints(x1, y1, x2-x1, y2-y1, oct, r, g, b, nx, ny, array);
    return 0;
}

int linePoints(int x0, int y0, int x2, int y2, int oct, USC r, USC g, USC b,
 int nx, int ny, USC* array)
{

    int dx, dy, incr1, incr2, d, x, y, x1, y1, xend, temp;

    
    x1 = 0;
    y1 = 0;
    if (oct == 2)
    {
        temp = x2;
        x2 = y2;
        y2 = temp;
    }
    if (oct == 7)
    {
        temp = x2;
        x2 = -y2;
        y2 = temp;
    }
    if (oct == 8)
    {
        y2 = -y2;
    }

    dx = x2 - x1;
    if (dx < 0) dx = -dx;
    dy = y2 - y1;
    if (dy < 0) dy = -dy;
    d = 2*dy - dx;
    incr1 = 2*dy;
    incr2 = 2*(dy - dx);
    if (x1 > x2)
    {
        x = x2;
        y = y2;
        xend = x1;
    }
    else
    {
        x = x1;
        y = y1;
        xend = x2;
    }
    plotLinePixel(x0, y0, x, y, oct, r, g, b, nx, ny, array);
    while (x < xend)
    {
        x = x + 1;
        if (d < 0)
        {
            d = d + incr1;
        }
        else
        {
            y = y + 1;
            d = d + incr2;
        }
        plotLinePixel(x0, y0, x, y, oct, r, g, b, nx, ny, array);
    }
}

int plotLinePixel(int x0, int y0, int x, int y, int oct, USC r, USC g, USC b,
 int nx, int ny, USC* array)
{

    if (oct == 1)
    {
        plotPixel(x0+x, y0+y, r, g, b, nx, ny, array);
        return 0;
    }
    if (oct == 2)
    {
        plotPixel(x0+y, y0+x, r, g, b, nx, ny, array);
        return 0;
    }
    if (oct == 7)
    {
        plotPixel(x0+y, y0-x, r, g, b, nx, ny, array);
        return 0;
    }
    if (oct == 8)
    {
        plotPixel(x0+x, y0-y, r, g, b, nx, ny, array);
        return 0;
    }
}

int readRGB1(FILE* inf, int* nx, int* ny, int* nlev)
{
    short junk[256], i;
    fread(junk, 2, 256, inf);
    *nx = junk[3];
    *ny = junk[4];
    *nlev = 255;
    return 0;
}

int readRGB2(FILE* inf, int nx, int ny, USC* array)
{
    USC cc;
    int i, j, k, c;
    for (k = 0; k < 3; k++) 
        {
        for (j = ny-1; j >= 0; j--) 
        {
            for (i = 0; i < nx; i++)
            {
                fread(&cc, 1, 1, inf);
                array[k+3*i+3*(nx)*j] = cc;
            }
        }
    }
    return 0;
}

int writeRGB(FILE* outf, int nx, int ny, int nlev, USC* array)
{
    USC cc;
    int i, j, k;
    short header[256];

    for (i = 0; i < 256; i++) 
    {
        header[i] = 0;
    }
    header[0] = 474;
    header[1] = 1;
    header[2] = 3;
    header[3] = nx;
    header[4] = ny;
    header[5] = 3;
    header[9] = nlev;
    header[11] = 3;

    fwrite(header, 2, 256, outf);

    for (k = 0; k < 3; k++)
    {
        for (j = ny-1; j >=0; j--)
        {
            for (i = 0; i < nx; i++)
            {
                cc = array[k+3*i+3*nx*j];
                fwrite(&cc, 1, 1, outf);
            }
        }
    }
    return 0;
}

int readXBM1(FILE* inf, int* nx, int* ny, int* nlev)
{
    int npix;
    char first[80];

    fgets(first, 80, inf);
    readXbmSize(first, &npix);
    *nx = npix;
    fgets(first, 80, inf);
    readXbmSize(first, &npix);
    *ny = npix;
    
    printf("XBM not yet implemented\n");
    return 0;
}

int readXBM2(FILE* inf, int nx, int ny, USC* array)
{
}

int writeXBM(FILE* outf, int nx, int ny, int nlev, USC* array)
{
}

#include <string.h>
int readXbmSize(char* line, int* npix)
{
    int count;
    int* val;
    char* nexttok;

    nexttok = strtok(line, " ");
    while (++count <= 3)
    {
        nexttok = strtok(0, " ");
    }
    sscanf(nexttok, "%d", npix);
    return 0;
}

char* getSuffix(char* filename)
{
    char* temp = strchr(filename, (int)'.');
    return ++temp;
}
    
