#_______________________________________________________________________#
# This file contains procedures concerned with the annotation of canvas #
# objects.                                                              #
#_______________________________________________________________________#

#=======================================================================#
# Identifies if there is a selected object, and gets information about it
# from its tags.

proc Annotate_Selected {} {

    global _Absolute_URL _Ask_for_Absolute_URL_path

# Calls procedure to get a list of attributes for selected object.
# This is a null string if no object is selected.
    set oldAttribs [Find_Attributes [.can.c1 gettags selected]]

    if {$oldAttribs == ""} {
        postMessage "No object was selected"
        return
    }

# Get any previuosly written comments or url for the object.  If these are
# not null-strings they will be inserted in the relevant entrybox.
    set Comments [lindex $oldAttribs 3]
    set URL [lindex $oldAttribs 4]

# If there is no absolute path for the url's, call procedure to allow user to
# input one.
    if {$_Ask_for_Absolute_URL_path} {
        if {$_Absolute_URL == ""} {
            set _Absolute_URL [Get_Absolute_URL_Path]
        }
    }

    if {$URL == ""} {
        set URL $_Absolute_URL
    }

    Configure_Menu_Buttons disabled

# Call procedure to create a new toplevel containing two entryboxes.
    getCommentsAndURL Comments URL

    Configure_Menu_Buttons normal

# Amend the attributes to hold the new values.
    set attributes [lrange $oldAttribs 0 2]
    lappend attributes $Comments
    lappend attributes $URL

# Remove the old-attribute-tag from the selected object and insert the new.
    .can.c1 dtag selected $oldAttribs
    .can.c1 addtag $attributes withtag selected

# If the object has a value in the url part of the attribute tag, then allow
# the user to test the url.
    Configure_Test_URLbutton
}

#=======================================================================#
# Checks for the existance of a window with same pathname and destroy it
# before creating the new one. 
# args    : 1. Any previously written comments for the object.
#           2. Any previously written url for the object.

proc getCommentsAndURL {Comments Url} {

    upvar 1 $Comments com
    upvar 1 $Url url

# Textvariables for the entryboxes.
    global comvar urlvar

# If these values already exist, set them to null.
    if {![info exists com]} {
        set com ""
    }
    if {![info exists url]} {
        set url ""
    }

# Set the textvars to hold the values.
    set comvar $com
    set urlvar $url

# Destroy the window if it already exists.
    if {[winfo exists .comurl]} {
        destroy .comurl
    }

# Call proc to create the toplevel allowing user to input information.
    set values [Create_Annotation_Toplevel]
# Set vars to appropriate return values.
    set url [lindex $values 0]
    set com [lindex $values 1]
}


#=======================================================================#
# Create the new toplevel allowing user to insert values for comments and
# url.
# returns : List of "url" and "comment" associated with object.
proc Create_Annotation_Toplevel {} {

# Textvariables for entryboxes
    global comvar urlvar

# Amessage to help user.
    set msg " Annotate the selected object with a comment and a URL.  These \
        are written to file. \n\
        The URL (which YOU should start with http:// or similar) can be \
        tested by pressing the TEST_URL button in the menu-bar.  It activates \
        your selected browser."

# Create the new toplevel with a message-widget at top.
    toplevel .comurl
    message .comurl.msg -text $msg -width 500
    pack .comurl.msg

# Create a frame containing the label and entry-widget for the url.
    frame .comurl.url
    pack .comurl.url -side top -expand yes -fill x
    label .comurl.url.lab -text URL -width 10 -anchor w
    pack .comurl.url.lab -side left 
    entry .comurl.url.ent -relief sunken
    .comurl.url.ent insert 0 $urlvar
    pack .comurl.url.ent -side left -expand yes -fill x
bind .comurl.url.ent <Return> {.comurl.com.ent icursor 0}

# Create a frame containing the label and entry-widget for the comment.
    frame .comurl.com
    pack .comurl.com -side top -expand yes -fill x
    label .comurl.com.lab -text Comments -width 10 -anchor w
    pack .comurl.com.lab -side left 
    entry .comurl.com.ent -relief sunken
    .comurl.com.ent insert 0 $comvar
    pack .comurl.com.ent -side left -expand yes -fill x 
    bind .comurl.com.ent <Return> {.comurl.bottom.ok invoke}
    frame .comurl.bottom
    pack .comurl.bottom -expand yes -fill x

# Ok button, bound to - user finished annotation, with new values.
    button .comurl.bottom.ok -text OK \
        -command {
            set comvar [.comurl.com.ent get]
            set urlvar [.comurl.url.ent get]
            set finished 1
        }
# Cancel button, bound to - user finished, no new values.
    button .comurl.bottom.cancel -text CANCEL \
        -command {
            set finished 1
        }
    pack .comurl.bottom.ok -side left -expand yes -fill x
    pack .comurl.bottom.cancel -side left -expand yes -fill x

# Wait for a button to be pressed, and destroy window returning values.
    tkwait variable finished
    destroy .comurl
    return [list $urlvar $comvar]
}
#=======================================================================#
# Identify if there are any canvas objects containing nothing in the
# url part of the attribute tag.
# returns : Index of the unannotated object. Or -1 if all objects are
#           annotated.

proc SelectFirstUnannotated {} {

    global _Absolute_URL

Configure_Cursor .can working

# Get a list of all canvas objects.
    set displayList [.can.c1 find all]

# Loop through identifying if this is a user created object.
    foreach object $displayList {
        set attributes [Find_Attributes [.can.c1 gettags $object]]

# If not, continue to next object.
        if {$attributes == ""} {
            continue
        }

# If object holds no value for the url, give it a tag so it can be identified,
# and return the index.
        set url [lindex $attributes 4]

        if {[string match $_Absolute_URL $url]} {
            .can.c1 addtag item$object withtag $object
            Configure_Cursor .can done
            return $object
        }
    }

    Configure_Cursor .can done
# Post a message telling user that all objects are annotated, and return an
# appropriate value.
    postMessage "All objects currently on display are annotated."
    return -1
}

