#____________________________________________________________________#
# This procedure is called in one of two ways :                      #
# 1. When a radiobutton is selected it is called with a string held  #
#    in the argument.                                                #
#    Depending on this string, various procedures are called to      #
#    bind handlers to the relevant sequence of events.               #
# 2. After a complete cycle in a particular sequence of events if    #
#    the Auto_Default checkbutton is switched to the off position,   #
#    or on invoking the "Default" menu-button, the  argument holds a #
#    null string ("").                                               #
#    In this case the procedure is called with an empty string.      #
#    This calls the relevant procedures to set up the default        #
#    bindings.                                                       #
#                                                                    #
# Arg : 1. Contains value of radiobutton variable "_Choice" if       #
#          procedure invoked from radiobutton.  Or an empty string   #
#          if call to reset default bindings.                        #
#____________________________________________________________________#

proc Set_Correct_Binds {whichFunction} {

# Set all canvas bindings to null before creating new ones.  This 
# removes the possibility of any "Rogue-Bindings".
    Nullify_All_Bindings
    Set_Message 1
        
# Set up switch command to call relevant procedures to set bindings
    switch $whichFunction {
        CIRCLE_RAD
                    -
        CIRCLE_BB
                    - 
        RECTANGLE
                    - 
        POLYGON {
# If "whichFunction" holds a string that matches any of the above, then the
# following commands are executed.

# Deselect any object currently selected.
            Deselect_Selected

# Call procedure to set up bindings for all functions which fall into the 
# "Create" category.  "whichFunction" controls which bindings are set.
            Create_Objects $whichFunction
        }
        COPY_SELECTED
                    -
        FLIP_ABOUT-X
                    -
        FLIP_ABOUT-Y
                    -
        MOVE_POINT
                    -
        ADD_POINT
                    -
        DELETE_POINT {
# If "whichFunction" holds a string that matches any of the above, but not the
# first four patterns, call procedure to set up bindings for all functions
# which fall into the "Edit" category.
# "whichFunction" controls which bindings are set.
            Edit_Objects $whichFunction
        }
        default {
# If the string in "whichFunction" does not match any of the strings 
# previously given, this command is executed by default.
# It invokes the procedure to set up the default canvas bindings.
            Which_Default
        }
    }
}

#============================================================================#

# Procedure called whenever a new function is invoked, or on completion of a
# complete cycle of a function (through "proc Set_Correct_Binds").
# Sets all canvas bindings used in MapMarker to null, therefore removing the
# possibility of having any rogue-bindings.

proc Nullify_All_Bindings {} {

    bind .can.c1 <ButtonRelease-1> {}
    bind .can.c1 <ButtonPress-1> {}
    bind .can.c1 <B1-Motion> {}
    bind .can.c1 <ButtonRelease-1> {}
    bind .can.c1 <Button-2> {}

    .can.c1 bind selected <ButtonPress-1> {}
    .can.c1 bind visualaid <ButtonPress-1> {}
    .can.c1 bind item <ButtonPress-1> {}

}

