#____________________________________________________________________________#
#|                                                                          |#
#| This file contains procedures to carry out all calculations, etc.        |#
#| Called when relevant.                                                    |#
#|                                                                          |#
#| Procedures :                                                             |#
#| 1. proc Deselect_Selected    - Deselects any currently selected object.  |#
#| 2. proc Remove_VisualAids    - Removes any visual-aids used while one of |#
#|                                the Create/Edit functions is being        |#
#|                                executed.                                 |#
#| 3. proc Calculate_Radius     - Calculates the coordinates of the visual- |#
#|                                aid when the "Circle_rad" option is being |#
#|                                executed.                                 |#
#| 4. proc Create_Canvas_Object - Draws the newly created object on the     |#
#|                                canvas.                                   |#
#|__________________________________________________________________________|#
#                                                                            #

#============================================================================#
proc Deselect_Selected {} {
    .can.c1 addtag item withtag selected

    .can.c1 dtag selected

    Remove_VisualAids
}
#============================================================================#

proc Remove_VisualAids {} {
    .can.c1 delete visualaid
}

#============================================================================#

# Called from "proc Drag_Motion" if the current "Create" selection is 
# "Circle_rad".
# The size of the circle is defined by the change in the x-coordinate from
# the startx value, so any change in the x-coordinate is transferred to the 
# y-coordinate.
# args :
#       1. Change in the x-coordinate while dragging with mouse-button-1.
#       2. The x-coordinate of the start position (from <ButtonPress-1>)
#       2. The y-coordinate of the start position (from <ButtonPress-1>)

proc Calculate_Radius {deltaX middleX middleY} {

# Set x1 and y1 to hold the coordinates of the top-left of the visaul-aid
    set x1 [expr ($middleX-$deltaX)]
    set y1 [expr ($middleY-$deltaX)]
# Set x2 and y2 to hold the coordinates of the bottom-right of the visaul-aid
    set x2 [expr ($middleX+$deltaX)]
    set y2 [expr ($middleY+$deltaX)]

# Return the coordinates of the visual-aid to caller
    return [list $x1 $y1 $x2 $y2]

}
#============================================================================#

# Draws the newly created object on the canvas.
# args :
#       1. The type of the object.
#       2. The coordinate-list of the object.
#       3. The draw-style of the object.
#       4. The colour to draw object in.
#       5. The line-width to draw object in.  Only used if style is outline.

proc Create_Canvas_Object {type coordinates style colour lWidth} {

    Remove_VisualAids

    if {$type == "polygon" || $type == "line"} {
        if {$style == "outline"} {
            set lastPoint [lrange $coordinates 0 1]

            eval ".can.c1 create line $coordinates $lastPoint -fill $colour \
                -width $lWidth -tags createdNow"
        } else {
            eval ".can.c1 create polygon $coordinates -fill $colour \
                -tags createdNow"
        }
    } else {
        if {$style == "outline"} {
            eval ".can.c1 create $type $coordinates -$style $colour -fill {} \
                -width $lWidth -tags createdNow"
        } else {
            eval ".can.c1 create $type $coordinates -$style $colour \
                -outline {} -tags createdNow"
        }
    }

    Auto_Select createdNow

}
#============================================================================#

proc Reset_To_Default_Binds {} {

    uplevel #0 {set _Choice ""}
    Set_Correct_Binds ""
}

