#____________________________________________________________________________#
#|                                                                          |#
#| This file contains procedures to carry out all calculations, etc.        |#
#| Called when relevant.                                                    |#
#|                                                                          |#
#| Procedures :                                                             |#
#| 1. proc Delete_Selected_Object   - Removes the selected object from the  |#
#|                                    canvas.                               |#
#| 2. proc LowerSelected            - Lowers the priority of the selected   |#
#|                                    object in the canvas display-list.    |#
#| 3. proc Create_Coordinate_Points - Creates coordinate-points for polygon |#
#|                                    type objects, when one of the "Edit-  |#
#|                                    Polygon" options is selected.         |#
#|__________________________________________________________________________|#
#                                                                            #

#============================================================================#

# Invoked from button "Delete_Selected" under "Edit" menu.
# Removes any currently selected object, and its bounding-box, etc.

proc Delete_Selected_Object {} {

# Remove the object and all visual-aids from canvas
    .can.c1 delete selected visualaid
}

#============================================================================#

# Procedure to lower an object 
#
# args :
#       1. The specified name of the currently loaded imagefile.
#          (empty string if no imagefile is currently loaded.)

proc LowerSelected {imgfile} {

# Get a list of all objects currently displayed on canvas.
    set displayList [.can.c1 find all]

# If there is currently an image loaded on the canvas, set var to 
# position above image.  Else to bottom of canvas' displayList.
    if {$imgfile != ""} {
        set belowThis [lindex $displayList 1]
    } else {
        set belowThis [lindex $displayList 0]
    }
# Lower selected object to specified position.
    .can.c1 lower selected $belowThis
}

#============================================================================#
# Create the coordinate points for polygon type objects when one of the 
# Polygon Point Edit" options is selected.

proc Create_Coordinate_Points {} {

    Remove_VisualAids
    set coords [.can.c1 coords selected]
    set lengthCoords [expr ([llength $coords]-2)]

# For each x,y coordinate create a point	
    for {set a 0} {$a<$lengthCoords} {incr a 2} {
        set x [lindex $coords $a]
        set y [lindex $coords [expr ($a+1)]]
        eval ".can.c1 create rectangle $x $y $x $y -width 5 \
            -outline green -tags visualaid"
    }
}

#============================================================================#
proc Check_Polygon {} {

    global _Choice

    set type [.can.c1 type selected]

    if {$type == "polygon" || $type == "line" } {
    } else {
        switch $_Choice {
            DELETE_POINT {
                set msg "Points can only be deleted from polygon objects. \n\n"
                Configure_Message_Widget $msg
            }
            ADD_POINT {
                set msg "Points can only be added to polygon objects. \n\n"
                Configure_Message_Widget $msg
            }
            MOVE_POINT {
                Set_Message 5
            }
        }
        after 2500
        return 0
    }

    if {$_Choice == "DELETE_POINT" &&
        [expr [llength [.can.c1 coords selected]]-1]<=8} {

        set msg "A polygon cannot have less than three points. \n\n"
        Configure_Message_Widget $msg
        after 2500
        return 0
    }
    return 1
}

