

#=========================================================================#
# If one of the "Grid" radiobuttons is selected, this procedure is called
# to create/delete the grid.
# args :
#       1. Integer value determining the line-spacing of grid.
#       2. The specified name of the currently loaded imagefile.
#          (empty string if no imagefile is currently loaded.)

proc Do_Grid {value imgfile} {

# If selection is not 'No_grid', delete any previous graph, and create
# a new one with the correct spacing.  If selection is 'No_grid',
# delete all graph

    if {$value!=0} {
        eval ".can.c1 delete graph"
        for {set a 0} {$a<=1000} {incr a $value} {
            eval ".can.c1 create line $a 0 $a 1000 -tags graph"
        }
        for {set a 0} {$a<=1000} {incr a $value} {
            eval ".can.c1 create line 0 $a 1000 $a -tags graph"
        }
    } else {
# Else delete any grid already on canvas
        eval ".can.c1 delete graph"
    }

    set displayList [.can.c1 find all]

# If there is currently an image loaded on the canvas, set var to 
# position above image.  Else to bottom of canvas' displayList.
    if {$imgfile != ""} {
        set belowThis [lindex $displayList 1]
    } else {
        set belowThis [lindex $displayList 0]
    }

# This lowers the grid to below the first canvas item draw by the user.
    catch {.can.c1 lower graph $belowThis}
}

#=========================================================================#
# Create a toplevel window consisting a message and an entry-box.
# args :
#       1. Message string to be displayed
#       2. String to be displayed in entry-box (defaults to null-string)

proc Get_Entry {msg {default ""}} {

    global entryval

# Destroy window if it exists
    if {[winfo exists .entry]} {
        destroy .entry
    }

# Make a toplevel window
    toplevel .entry 
# Message telling user what is required as input in the entry-widget.
    message .entry.msg -text $msg -width 500
    pack .entry.msg
# Entry widget allowing user to input data.
    entry .entry.entry -relief sunken 
    pack .entry.entry -side top -expand yes -fill x
# Frame for buttons.
    frame .entry.bottom 
    pack .entry.bottom -expand yes -fill x
# OK and Cancel buttons.
    button .entry.bottom.ok -text OK \
        -command {set entryval [.entry.entry get]}
    button .entry.bottom.cancel -text CANCEL \
        -command {set entryval ""} 
    pack .entry.bottom.ok -side left -padx 120
    pack .entry.bottom.cancel -side right -padx 120
# If input focus is the entry-widget, pressing <Return> returns $entryval
# to calling procedure.
    bind .entry.entry <Return> {.entry.bottom.ok invoke}
# Insert the default string in entry-widget
    .entry.entry insert 0 $default
# Wait for OK to be invoked, and destroy window.
    tkwait variable entryval
    destroy .entry
# Return string to caller.
    return $entryval
}

#=========================================================================#

proc Get_URL_Of_Selected {} {

    set url ""
    set attrList [Find_Attributes [.can.c1 gettags selected]]

    if {$attrList != ""} {
        set url [lindex $attrList 4]
    }
    return $url
}


#=========================================================================#
# Called when button "Test_URL" is invoked.
proc Test_Selected_Url {} {

    global env BROWSER  browsepid

    set attrList [Find_Attributes [.can.c1 gettags selected]]

    if {$attrList == ""} {
        postMessage "There is currently no selected object."
    } else {

        set url [Get_URL_Of_Selected]

        if {$url == ""} {
            postMessage "The selected item has not been fully annotated"
        } else {
            set browsepid [Spawn_Browser $BROWSER $url $browsepid]
            set allPaths \
                [list file options create edit attributes html_help help]

            if {$BROWSER != "Mosaic"} {                
                foreach path $allPaths {
                    .can.menu.$path configure -state disabled
                }
            }

# dumb browsers have to be restarted each URL
            if {![Resident_Browser $BROWSER $browsepid] } {
               postMessage "A browser will be displayed shortly. When you have \
                    finished the test, press OK here, rather than quitting the \
                    browser"
                exec kill -9 $browsepid
                foreach path $allPaths {
                    .can.menu.$path configure -state normal
                }
# Mosaic and others can remain running
            } else {
            }
        }
    }
}


#=========================================================================#
proc Spawn_Browser {browser url browsepid} {


# if we haven't kicked off a browser, or it isn't Mosaic kick one off
# (the form 'Mosaic' is critical for the /tmp/Mosaic.<pid>)
    if {$browsepid == -1 || $browser != "Mosaic"} {
        set browsepid [exec $browser $url &]
        return $browsepid
    }

# we already have a Mosaic, so don't waste time restarting it...
# write the URL to a file /tmp/Mosaic.<pid>
    set f [open /tmp/$browser.$browsepid w]
    if {$f == ""} {
        postMessage "Error opening temporary Mosaic file"
        return -1
    }
    puts $f "goto"
    puts $f "$url"
    close $f

    exec kill -USR1 $browsepid
    return $browsepid
}

#=========================================================================#
proc Resident_Browser {browser browsepid} {
# do we have a browser already running (at present only allowed w. Mosaic)

    if {$browser != "Mosaic" || $browsepid == -1} {
        return 0
    }
    return 1
}

#=========================================================================#
proc Configure_Cursor {win state} {

    switch $state {
        working {
            $win configure -cursor watch
        }
        done {
            $win configure -cursor {left_ptr red}
        }
    }
}

#=========================================================================#
proc Configure_Test_URLbutton {} {

    if {[Get_URL_Of_Selected] != ""} {
        .can.menu.test configure -state normal
    } else {
        .can.menu.test configure -state disabled
    }
}

#=========================================================================#
proc Set_To_New {} {

    global _InputImageFile _MapFile _ImageFile _Defaulthtml _Absolute_URL

    .can.c1 delete all

    set _InputImageFile ""
    set _MapFile ""
    set _ImageFile ""
    set _Defaulthtml ""
    set _Absolute_URL ""

    Reset_To_Default_Binds
}
#=========================================================================#
# If the "Single_Edit_Mode" checkbutton is off, this procedure resets the
# canvas bindings to those of the selected function.
proc Reset_To_Function_Binds {} {

    global _Choice

    Set_Correct_Binds $_Choice
}
#=========================================================================#
proc Configure_Menu_Buttons {state} {

    set menus [list file mode options create edit attributes]

    foreach menu $menus {

        .can.menu.$menu configure -state $state

    }
}
#=========================================================================#
proc Which_Mode {mode} {

    global f tfile _NewImage

    Set_Message 1

    switch $mode {
        map {
            if {[info exists f]} {
                close $f
                unset f
            }
            set _NewImage ""
            Reset_To_Default_Binds
            return
        }
        maptoimage {
            Map_To_Image
        }
    }
}

