############################################################################
# This file contains all procedures concerned with on-line help.           #
############################################################################

#--------------------------------------------------------------------------#
# Create a new toplevel window consisting :
#     1. Listbox-widget
#     2. Message-widget
# Insert a list of all MapMarker functions in the list-box and set up 
# buttons to view help-message of selected element and destroy window.

proc Help_File_Toplevel {} {

# Create the new toplevel, set its size and title
    toplevel .help
    wm geometry .help 700x400
    wm title .help Help

# Create 2 frames - one for the listbox-widget and one for the message-widget
    frame .help.f1 -height 300 -width 300
    frame .help.f2 -height 300 -width 400

# Pack these next to eachother in the window
    pack append .help .help.f1 {left filly} \
                      .help.f2 {left filly}

# Create a scrollbar so the user can adjust the displayed portion of listbox
    scrollbar .help.f1.scroll1 -command ".help.f1.l1 yview"

# Create the listbox and make it so only one object can be selected at a time
    listbox .help.f1.l1 -relief sunken -geometry 27x15 \
        -yscroll ".help.f1.scroll1 set"
    tk_listboxSingleSelect .help.f1.l1

# Create the button to select an option for help, and set its command to call
# the procedure to change the displayed message.
    button .help.f1.b1 -relief raised -text Select \
        -command {
                set helpChoice [.help.f1.l1 curselection]
                HelpFile_ShowMessage $helpChoice
        }

# Create the button used to remove the window from the screen
    button .help.f1.b2 -relief raised -text OK -command "destroy .help"

# Pack the widgets in the frame on the left
    pack append .help.f1 .help.f1.scroll1 {right filly} \
                         .help.f1.l1 {top expand fill} \
                         .help.f1.b1 {left} \
                         .help.f1.b2 {right}

# Set a list to hold all values of strings to be inserted in the listbox
    set listBoxElements [list \
        New Load_Image Load_Map Save Save_Map_As Save_Image_As \
        "Modify Mapfile" "Map -> Image" Quit \
        "Default Edit Mode" Single_Edit_Mode "Set Default_HTML" \
          "Set Absolute_URL path" "Ask_for_Absolute_URL_Path" \
          "Annotate Selected" "Select First Unannotated" Force_Annotation \
          Auto_Raise Grid_Over_Picture\
        "Create Circle (r)" "Create Circle (bb)" "Create Rectangle" \
          "Create Polygon" \
        Copy_Selected Flip_About-x Flip_About-y Move_Point Add_Point \
          Delete_Point "Raise Selected" "Lower Selected" Delete_Selected \
        Style Colour Line_Width Test_URL]

# Loop through list insert each element into the listbox-widget
    foreach element $listBoxElements {
        .help.f1.l1 insert end $element
    }
# Set the 'default help-message'
    set helpmess "By pressing mouse-button-1 on an object and clicking \
        \"Select\" gets relevant help message. \n\
        Alternatively, Double-click Mouse button1 on required listbox \
        element.\n\  Click OK when finished"

# Create the message and display it in the frame on the right
    message .help.f2.m1 -text $helpmess -width 390
    pack append .help.f2 .help.f2.m1 {frame nw}

    bind .help.f1.l1 <Double-1> {
        set selected [Get_Listbox_Selection %W %y]
        HelpFile_ShowMessage $selected
    }
}

#--------------------------------------------------------------------------#
# This procedure amends the textual-string displayed in the 
# message-widget in the help-window.
# args :
#       1. Contains the element reference of the selected item.

proc HelpFile_ShowMessage {listboxIndex} {

    set newHelpMessage "By pressing mouse-button-1 on an item and clicking \
        select gets relevant help message. \n\
        Alternatively, Double-click Mouse button1 on required listbox \
        element.\n\  Click OK when finished"
    switch $listboxIndex {
        New {
# Help message for New
            set newHelpMessage " Clears the canvas. \n\
                Allowing a fresh map or image to be loaded."
        }
        Load_Image {
# Help message for Load_Image
            set newHelpMessage " Brings up a file-browser.  This is very \
                similar to the one found in Mosaic. \n\
                Browse through and find the required image-file.\n\n\
                Current acceptable forms for images are : \n\
                1. gifs. (Converted to b/w xbms)\n\
                2. rppms. \n\
                3. xbms. "
        }
        Load_Map {
# Help message for Load_Map
            set newHelpMessage " Brings up a file-browser.  This is very \
                similar to the one found in Mosaic. \n\
                Browse through and find the required map-file \
                (NCSA format only). \n\
                If the map-file was created using Map_Marker, you may be \
                offered the option of loading the relating image-file also."
        }
        Save {
            set newHelpMessage " Depending on the current \"Mode\" of \
                operation - Selected from the Mode-menu - the current state \
                of the canvas is saved to file. \n\n\
                1.  If the Modify_Mapfile Mode is selected, all objects on the \
                canvas will be saved to a mapfile. \n\
                2.  If the Map -> Image Mode is selected, then the mapfile \ 
                will be saved as normal, and a modified version of image will \
                be saved.  This will contain mark-ups relative to the created \
                objects. \n\n\
                See help messages for the Mode Menu for more information."
           

        }
        Save_Map {
# Help message for Save_Map
            set newHelpMessage " If the current mapfile has been loaded from \
                a file, or saved previously under a specified name, then the \
                current state of the canvas is saved under that name. \n\
                If this button is pressed with no filename to save to, \
                the operation for Save_Map_As is executed. \n\
                An additional request is for a default_html to point to.\n\
                Finally, an extra is that if there is currently an image-file \
                specified, the filename will be saved in the mapfile. \
                Then if the mapfile is reloaded the option will be offered \
                to reload the image-file also."
        }
        Save_Map_As {
# Help message for Save_Map_As
            set newHelpMessage " Brings up a file-browser.  \n\
                Move to the directory you wish to save the mapfile in, and \
                insert the name of the new file into the \"Filename\" \
                entrybox.  Press OK and the file will be saved. \n\
                If there is currently no default_html file \
                specified, the option of naming one will be offered.\
                See the \"Set Default_HTML\" help message to see how this is \
                done.."
        }
        Save_Image_As {
            set newHelpMessage " Brings up a file-browser.  \n\
                Move to the directory you wish to save the modified image in, \
                and insert the name of the new file into the \"Filename\" \
                entrybox.  Press OK and the file will be saved. \n\n\
                This will only be saved if the Mode option is set to \n\
                \" Map -> Image \"."
        }
        {Modify Mapfile} {
            set newHelpMessage " This option is set as the default. \n\
                This is used to tell MapMarker what is to be saved whenever \
                the Save button is invoked.  With the Mode option set to this \
                a Mapfile will be saved under the specified name when the Save \
                button is invoked."
        }
        {Map -> Image} {
            set newHelpMessage " This option is new.  If the Mode option is \
                set to this, then a mapfile will be saved as normal, and a \
                modified version of the image is saved also. \n\
                The modified Image will contain mark-ups corresponding to the \
                areas you have marked up using MapMarker."
        }
        Quit {
# Help message for Quit
            set newHelpMessage " Does exactly that.\n\
                Just in case this is accidentally invoked, a small window \
                will appear asking if this is what you really do wish to \
                quit. \n\
                \n Just press Yes or No."
        }
        {Default Edit Mode} {
# Help message for "Default Edit Mode"
            set newHelpMessage " This button returns the canvas bindings to \
                the Default settings. \n\
                These are : \n\
                1. Select A New Object \n\
                2. Deselect the Currently Selected Object \n\
                3. Move the Selected Object \n\
                4. Scale the Selected Object"
        }
        Single_Edit_Mode {
# Help message for Single_Edit_Mode
            set newHelpMessage " If this checkbutton is switched on, then only \
                one radiobutton edit option will be executed at a time. After \
                completing one operation, the canvas bindings will \
                automatically be reset to the defaults. \n\
                If you wish to execute the same function repeatadly - such as \
                creating a circle, or making multiple copies of an object - \
                switching this button off will leave the canvas bindings as \
                they are until the \"Default Edit Mode\" button is invoked." 
        }
        {Set Default_HTML} {
# Help message for "Set Default_HTML"
            set newHelpMessage " Brings up a new window containing an \
                entry-box.  Insert the name of the default_html file you \
                would like the background of the image to point to and \
                press OK. \n\n\
                If the Absolute_URL path has been set this will be displayed \
                in the entrybox, and the filename should be appended on."
        }
        {Set Absolute_URL path} {
# Help message for "Set Absolute_URL path"
            set newHelpMessage "Brings up a new window containing an \
                entry-box. \n\
                Into this you must insert the \"Absolute\" pathname of the \
                directory containing the html files for the image. \n\n\
                This should be in the form :- \n\
                http://server.net.address/dir_name/ \n\n\
                The directory_name should include all directories in the path \
                leading up to and including the directory containing the files \
                you wish to point to. This should also include the last \"/\" ." 
        }
        Ask_for_Absolute_URL_Path {
# Help message for "Ask_for_Absolute_URL_Path"
            set newHelpMessage " If this checkbutton is in the \"ON\" position \
                whenever an object is created in a new mapfile, before \
                annotation is carried out a window will appear asking you to \
                insert the pathname of the directory containing the relevant \
                html files. \n\n\
                This will then be inserted into the URL entry box of all \
                objects. \n\n\
                It can be turned off for cases where the documents to be \
                pointed to do not live in one directory."
        }
        {Annotate Selected} {
# Help message for Annotate Selected
            set newHelpMessage " If this button is pressed and there is \
                currently a selected object on the canvas, a window will be \
                created containing two entry-boxes. \n\
                If the object has previously been annotated to any degree, \
                this information will be contained in the relevant entry-box. \
                If not then the entry-boxes will be empty. \n\
                This can now be edited to hold the required information."
        }
        {Select First Unannotated} {
# Help message for Select First Unannotated
            set newHelpMessage " If this button is pressed, all objects on \
                the canvas will be checked to see if it holds a value for \
                a URL. \n\
                The first object in the canvas' display-list with no URL-value \
                will automatically be selected, and a message-box will come \
                up to confirm this.  Press OK, then the \"Annotation\" \
                window will be created allowing annotation to be carried out."
        }
        Force_Annotation {
# Help message for Force_Annotation
            set newHelpMessage " If this checkbutton is switched on, \
                each time a new object is created the \"Annotation\" window \
                will be called.  This means that no object can escape being \
                annotated. \n\
                It is switched to the \"ON\" state by default."
        }
        Auto_Raise {
# Help message for Auto_Raise
            set newHelpMessage " If this checkbutton is switched on, each \
                time a new object is selected it will automatically be raised \
                to the top of the canvas' display list.  In the off position, \
                all objects will hold relative positions in the display \
                list. \n\
                It is switched to the \"ON\" state by default."
        }
        Grid_Over_Picture {
# Help message for Grid_Over_Picture
            set newHelpMessage " A cascade menu containing a series of \
                radiobuttons.  The \"No_Grid\" button is set as the \
                default. \n\
                This is intended purely as an aid to help the user in \
                creating objects to completely envelop areas."
        }
        {Create Circle (r)}
 {
# Help message for Create Circle (r)
            set newHelpMessage "Pressing Mouse-button-1 identifies a \
                point for the centre of the circle.  Dragging in the \
                x-direction increases/decreases size.  Release button \
                to draw the object."
        }
        {Create Circle (bb)} {
# Help message for Create Circle (bb)
            set newHelpMessage "Pressing Mouse-button-1 identifies a point \
                for either the top-left or the bottom-right corner of a \
                bounding-box in which the circle will be drawn. Dragging in \
                the x-direction increases/decreases size.  Release button to \
                draw the object."
        }
        {Create Rectangle} {
# Help message for Create Rectangle
            set newHelpMessage "Pressing Mouse-button-1 identifies a point \
                for either the top-left or the bottom-right corner of a \
                bounding box in which the rectangle will be drawn. Dragging \
                in the x/y-directions increases/decreases size.  Release \
                button to draw the object."
        }
        {Create Polygon} {
# Help message for Create Polygon
            set newHelpMessage "Pressing Mouse-button-1 puts a point at the \
                location of the cursor.  This is done as many times as \
                required, with the last point being drawn with \
                Mouse-button-2.  This closes the polygon and draws it."
        }
        Copy_Selected {
# Help message for Copy_Selected
            set newHelpMessage " Pressing Mouse-button-1 in the selected \
                object displays an outline of it. This also creates a \
                copy of the object over the original. \n\
                Dragging with Mouse-button-1 still pressed allows the \
                outline to be dragged across the canvas. \n\
                Releasing Mouse-button-1 calculates the change in position \
                for the copy and moves the copy.  This is only executed if \
                the mouse has been dragged across the canvas.  If no motion \
                has occurred then the copy is deleted."
        }
        Flip_About-x {
# Help message for Flip_About-x
            set newHelpMessage " This function is carried out by the user \
                pressing Mouse-button-1 in one of the points on the \
                bounding-box of the selected object. \n\
                The flip occurs using the chosen point as a centre for the \
                flip.  With this option, there is no change in the coordinate \
                y-values."
        }
        Flip_About-y {
# Help message for Flip_About-y
            set newHelpMessage " This function is carried out by the user \
                pressing Mouse-button-1 in one of the points on the \
                bounding-box of the selected object. \n\
                The flip occurs using the chosen point as a centre for the \
                flip.  With this option, there is no change in the \
                coordinate x-values."
        }
        Move_Point {
# Help message for Move_Point
            set newHelpMessage " This is only used on polygon type objects. \n\
                If this button is selected, the bounding-box and its points \
                will be deleted and coordinate points for the object will be \
                shown. \n\
                Any of these points can be picked up by pressing \
                Mouse-button-1 in it.  Keeping Mouse-button-1 pressed and \
                dragging the cursor across the canvas moves the chosen point \
                with it.  Move it to the required location and release the \
                button to put the point down."
        }
        Add_Point {
# Help message for Add_Point
            set newHelpMessage " This is only used on polygon type objects. \n\
                If this button is selected, the bounding-box and its points \
                will be deleted and coordinate points for the object will be \
                shown. \n\
                Press Mouse-button-1 in a point creates a point to be \
                inserted after the selected point in the coordinate list of \
                the polygon.  Dragging with Mouse-button-1 draws a line \
                connecting the point with the one after it, through the \
                current cursor location. \n\
                Releasing the mouse-button creates the additional point at \
                the current cursor location."
        }
        Delete_Point {
# Help message for Delete_Point
            set newHelpMessage " This is only used on polygon type objects. \n\
                If this button is selected, the bounding-box and its points \
                will be deleted and coordinate points for the object will be \
                shown. \n\
                Pressing Mouse-button-1 in any of the coordinate points will \
                delete that point and redraw the amended polygon. \n\
                NB// A polygon cannot have less than three coordinate points."
        }
        {Raise Selected} {
# Help message for Raise Selected
            set newHelpMessage "Raises the currently selected object \
                above all others on the canvas."
        }
        {Lower Selected} {
# Help message for Lower Selected
            set newHelpMessage "Lowers the currently selected object below \
                all others, to a position just above the photo/bitmap."
        }
        Delete_Selected {
# Help message for Delete_Selected
            set newHelpMessage " If there is currently a selected object on \
                the  canvas and this button is pressed, the object will be \
                deleted. \n\n\
                NB// There is no \"Undo\" button so be careful."
        }
        Style {
# Help message for Style
            set newHelpMessage " This is one of the attributes of the \
                objects.  It controls the style in which an object is \
                displayed on the canvas. \n\
                FILL        - draws the object fully coloured in. \n\
                OUTLINE - draws the object as an outline."
        }
        Colour {
# Help message for Colour
            set newHelpMessage " This is one of the attributes of the objects. \
                \n\ It controls the colour in which an object is drawn."
        }
        Line_Width {
# Help message for Linewidth
            set newHelpMessage " This is one of the attributes of the objects. \
                \n\ If an object is drawn in \"Outline\" style, this controls \
                the width of the outline."
        }
        Test_URL {
# Help message for Test_URL
            set newHelpMessage " This button only becomes active if there is \
                currently a selected object on the canvas and that object has \
                been annotated to give a URL. \n\
                Pressing this button spawns your chosen browser with the file \
                that is pointed to."
        }

    }
# Reconfigure the help message-widget
    .help.f2.m1 configure -text $newHelpMessage -width 390
}

#--------------------------------------------------------------------------#
# This procedure sets the string to be displayed in the message-widget.
# This is only called for messages that are called from several places.
# args :
#       1. Value by which relevant message-string is set.

proc Set_Message {index} {

    global _Mode

    switch $index {
        1 {
            if {$_Mode == "map"} {
                set message "\n\nCreating a Mapfile."
            } else {
                set message "\n\nCreating a Mapfile and Marking a new image."
            }
        }
        2 {
            set message " <ButtonPress-1>     - Identifies top-left or \
                bottom-right corner. \
                \n <B1-Motion>           - To the opposite corner. \
                \n <ButtonRelease-1> - Creates object."
        }
        3 {
            set message "There is currently no selected object select one \
                and try again. \n"
        }
        4 {
            set message "A polygon cannot have less than three coordinate \
                points. \n"
        }
        5 {
            set message "Use scaling facility for rectangles and circles. \n\n"
        }
    }
 
    Configure_Message_Widget $message
}

#--------------------------------------------------------------------------#
# Procedure reconfigures the message-widget on the bottom of the main
# MapMarker window to contain a relevant piece of information to aid 
# the user.
# args :
#       1. String to display in message widget.

proc Configure_Message_Widget {newMessage} {

    .can.mess.m configure -text $newMessage -aspect 1000

    update
} 

#--------------------------------------------------------------------------#
# General purpose box that posts a message.
# args :
#       1. Message to display in the message-widget.

proc postMessage {msg} {
    global messageflag

    toplevel .message
    message .message.msg -text $msg -width 500
    pack .message.msg
    frame .message.bottom 
    pack .message.bottom -expand yes -fill x
    button .message.bottom.ok -text OK \
        -command {set messageflag 1; destroy .message}
    pack .message.bottom.ok
    tkwait variable messageflag
}

proc About {} {

    if {![winfo exists .about]} {
        set msg \
            " MapMarker is written by me - Gudge - as a project during an \
            industrial year at Glaxo Research and Development (UK). \n\
            The project was supervised by Peter Murray-Rust, and is part of an \
            initiative at Glaxo to develop high quality tools for \
            bioinformatics. \n\n\
            We have strived to make this software easy to use, and we hope you \
            find it useful.  If you find any bugs, have any comments or even \
            just want to say Hi, please don't hesitate to contact me. \n\n\
            This software is supplied as FreeWare to the Internet Community \
            for personal and internal business and research purposes.\n\n\
            It may not be distributed as part of a commercial package nor used \
            to create data which is sold commercially without permission of \
            the authors and Glaxo. \n\n\
            Gudge (akc22282@ggr.co.uk)."

        toplevel .about
        wm title .about "About MapMarker"
        message .about.msg -text $msg -width 500
        pack .about.msg -side top -fill both
        frame .about.f
        pack .about.f -side top -fill x
        button .about.f.ok -text OK -command "destroy .about"
        pack .about.f.ok -side top
    }
}
