###############################################################################
# This file contains all procedures concerned with loading image-files onto   #
# the canvas, and loading and saving of mapfiles.                             #
###############################################################################

#=============================================================================#
# Identifies if the file selected has one of the accepted suffixes.
# args :
#       1. Name of selected file

proc Query_Image_Format {filename} {

global _ImageFile

# Get extension part of file.
    set extension [file extension $filename]

# If extension is one of the recognized format suffixes, call proc to
# load the imagefile.  If not, post an appropriate error message.
    switch $extension {
        .xbm
            -
        .rppm
            -
        .gif {
            Image_Loader $filename
        }
        default {
            postMessage "Cannot read Image-files with extension : $extension"
        }
    }
}

#=============================================================================#
# Checks for existance and readability of selected file, and if both are ok
# Performs neccessary functions and loads the image.
# args :
#       1. Name of new imagefile to be loaded.

proc Image_Loader {newImage} {

    global _ImageFile env

    set _ImageFile $newImage

# Configure the cursor display, and insert a message into message widget.
    Configure_Cursor .can working
    Configure_Message_Widget "\nLoading image file :\n $_ImageFile"

    if {![file exists $_ImageFile]} {
        postMessage "Couldn't find file : \n $_ImageFile"
        Configure_Cursor .can done
        return
    }

    set xmax ""; set ymax ""

# Split the pathname of the file into its root and extension parts.
    set root [file rootname $_ImageFile]
    set extension [file extension $_ImageFile]

# If the file is not a "*.rppm", use imfile to  find the Resolution field.
# From this, identify the image size.
    if {$extension != ".rppm"} {
# Call procedure to identify the maximum x and y coordinates for image-file.
        set maxCoords [Find_Resolution $_ImageFile]
        set xmax [lindex $maxCoords 0]
        set ymax [lindex $maxCoords 1]
    }
 
# If file is "*.gif", do the best we can at present.
    if {$extension == ".gif"} {
        if {[info commands photo] == ""} {
            postMessage "Converting gif to xbm - result may be useless! \n\
                Press OK to execute conversion."
            exec imconv $_ImageFile $root.xbm
            set extension .xbm 

            postMessage "Conversion completed. \n\
                If it's horrid, you'll have to make a *.xbm or *.rppm by hand."
            set _ImageFile $root.xbm
        } else {
            postMessage "Converting gif to rppm. \n\
                Press OK to execute conversion."
            Configure_Message_Widget "\nLoading image file :\n $root.rppm"
            exec imconv $_ImageFile $root.rgb

            set qf [open /usr/tmp/quitfile$$ w]
            puts $qf "QUIT"
            exec $env(EDITIMAGE) $root.rgb $root.rppm < /usr/tmp/quitfile$$

            exec rm $root.rgb
            set extension .rppm
            set _ImageFile $root.rppm
        }
    }

# Depending on the file type (extension), execute appropriate command.
    switch $extension {
        .xbm {
            .can.c1 delete all
            set xcentre [expr ($xmax/2)]
            set ycentre [expr ($ymax/2)]
            .can.c1 create bitmap $xcentre $ycentre -bitmap @$_ImageFile
        }
        .rppm {
            .can.c1 delete all
            set xmax 1000; set ymax 1000
            set photo_id .can.c1-[.can.c1 create photo 0 0 $xmax $ymax]
            readppm $photo_id $_ImageFile
            postMessage " If the image appears *blotchy*, don't fear. \n\
                It seems to be a feature of the photo-widget. \n\
                Your original image is not affected."
        }
    }
# Reconfigure cursor to red, left_ptr.
    Configure_Cursor .can done
}
#=============================================================================#
# Uses "imfile" on the command line to determine the maximum x and y 
# coordinates of the image file.
# args :
#       1. Name of imagefile.

proc Find_Resolution {imageFile} {

# Gets info on file and splits each line into a list element.
    set infoList [exec imfile -long $imageFile]
    set infoLines [split $infoList \n]

# Loop through list until the "Resolution:" field is located.  From this set
# maximum x and y values.
    foreach line $infoLines {
        if {[lindex $line 0] == "Resolution:"} {
            set xmax [lindex $line 1]
            set ymax [lindex $line 3]
            break
        }
    }

# If either of these are a null string, system has fallen-down.
# Post appropriate error message, and return a null string to caller.
    if {$xmax == "" || $ymax == ""} {
        set msg "ERROR : imfile cannot find image limits in : \
            _InputImageFile.\n\n\
            imfile output is : $infoList."
        postMessage $msg
        return
    }
# Else return the coordinate values.
    return [list $xmax $ymax]
}

#=============================================================================#
# Ensures every canvas object has full annotation, and writes relevant
# information to file, specified by global "_MapFile" 

proc Save_Map {} {

    global _MapFile _Defaulthtml _ImageFile

    Configure_Cursor .can working

# Loop through all canvas objects checking for url.  If other than "-1"
# returned, then the object without a url value is selected for annotation.
    set queryUnannotated [SelectFirstUnannotated]

    if {$queryUnannotated != -1} {
        Configure_Cursor .can done
        Auto_Select item$queryUnannotated
        postMessage "There are currently unannotated objects displayed on the \
            canvas. \nThe first of which has been selected."
        Annotate_Selected
        return
    }

# The mapfile will fall over if no default-html is specified.
    if {$_Defaulthtml == ""} {
        set _Defaulthtml [Get_Default_html]
    }

    if {$_MapFile == ""} {
        postMessage "No mapfile has been specified. \n\
            Press OK and a browser will be displayed.  Insert a filename and \
            press OK, or Cancel to quit browser."
        .can.menu.file.men invoke 4
        return
    }

# Make sure the mapfile can be opened for writing.
    if {[catch "set f [open $_MapFile w]" msg] != 0} {
        postMessage "Cannot open mapfile ($_MapFile) because : \n $msg"
        return
    }

    Configure_Message_Widget "\n\n Saving Mapfile."

# MapMarker writes the Image-filename to the mapfile as a comment.
    if {![info exists _ImageFile] || $_ImageFile == ""} {
        postMessage "No image file has been specified, so no name will be \
            written to file."
    } else {
        puts $f "# _IMAGEFILE $_ImageFile"
    }

# Puts the default-html as the first non-comment line
    puts $f "default $_Defaulthtml"

# Set a list containing an index for each canvas object.
    set canvasObjects [.can.c1 find all]

# Loop through list elements setting vars to the correct values required
# by imagemap for the mapfile.
    foreach object $canvasObjects {

        set attributes [Find_Attributes [.can.c1 gettags $object]]

        if {$attributes == ""} {
            continue
        }

        set type [lindex $attributes 0]
        set colour [lindex $attributes 1]
        set lineWidth [lindex $attributes 2]
        set comments [lindex $attributes 3]
        set url [lindex $attributes 4]
        set coords [.can.c1 coords $object]

# The difference between tcl-values and mapfile-values are amended here.
        set fileValues [Amend_Attribute_Values $type $coords]
        set type [lindex $fileValues 0]
        set coords [lindex $fileValues 1]

# Loop through coordinates replacing any decimals with integer values
        set index1 0
        foreach co $coords {
            if {[string first . $co] == -1} {
            } else {
                set xy [split $co ,]
                set index2 0
                foreach el $xy {
                    if {[string first . $el] != -1} {
                        set strInd [string first . $el]
                        set el [string range $el 0 [expr $strInd-1]]
                        lvarpop xy $index2 $el
                    }
                    incr index2
                }
                set xy [join $xy ,]
                lvarpop coords $index1 $xy
            }
            incr index1
        }


# Puts the lines in the mapfile.
        puts $f "# _COLOUR $colour _LINEWIDTH $lineWidth \
            # _COMMENTS {$comments}"
        puts $f "$type $url $coords"
    }
# Close the mapfile once all objects have been written to file.
    close $f
# Reconfigure the cursor display.
    Configure_Cursor .can done
    Set_Message 1
}

#=============================================================================#
# The ambiguities between how tcl and the mapfile refer to the object-type
# and its coordinate list are amended here.
# args :
#       1. The type of the object.
#       2. The coordinate list of the object.

proc Amend_Attribute_Values {typ coordinates} {

# Set "typ" to mapfile version of tcl-type
    switch $typ {
        oval {
            set typ circle
        }
        line
            -
        polygon {
            set typ poly
        }
        rectangle {
            set typ rect
        }
    }

# The mapfile requires circle coordinates as :
#     x-centre,y-centre x-edge,y-edge.
# Tcl requires the top-left and bottom-right coordinates of a bounding-
# box which the circle will fit into.
# The coordinates are changed here.
    if {$typ == "circle"} {

        set dx [Calculate_Difference \
            [lindex $coordinates 0] [lindex $coordinates 2]]
        set x1 [expr ([lindex $coordinates 0]+$dx/2)]

        set dy [Calculate_Difference \
            [lindex $coordinates 1] [lindex $coordinates 3]]
        set y1 [expr ([lindex $coordinates 1]+$dy/2)]

        set coordinates [list $x1 $y1 $x1 [lindex $coordinates 3]]
    }

# Mapfile coord-list : x1,y1 x2,y2 .....xN,yN
# Tcl coord-list     : x1 y1 x2 y2 .....xN yN
# A comma is inserted here between corresponding x and y coordinates.
    set nco 0
    set coordString ""

    foreach coord $coordinates {
        set coordString "$coordString$coord"

        if {$nco%2 == 0} {
            set coordString "$coordString,"
        } else {
            set coordString "$coordString "
        }
        incr nco
    }

# Return amended values to caller.
    return [list $typ $coordString]
}

#=============================================================================#
# Calls "proc Get_Entry" to get the user to input a default-html for the
# mapfile.

proc Get_Default_html {} {

    global _Absolute_URL

    return [Get_Entry "Please enter name of Default html file to use for \
        imagemap." $_Absolute_URL]
}



proc Get_Absolute_URL_Path {} {

    postMessage "After Pressing OK a new window will be brought up, \
        containing an entry box.  You must insert the absolute path of the \
        URL's for the mapfile into here. \n\
        Any object specific url's will then be appended onto this when saving \
        to file."

    return [Get_Entry "Please enter the absolute path of the directory \
        containing the html files. \n In the form : \n\
        http://server.net.address/dir_name/"]

}
#=============================================================================#
# Stores the lines of the mapfile as list elements.  Loops through these 
# identifying what eah line is, and calling procs to execute commands.

proc Read_Mapfile {} {

    global _MapFile _Defaulthtml _ImageFile

    if {![file exists $_MapFile] || ![file readable $_MapFile]} {
        postMessage "ERROR : \n Error in accessing file : $_MapFile"
        return
    }

    if {[file extension $_MapFile] != ".map"} {
        postMessage "Unable to read mapfiles with extension : \
            [file extension $_MapFile]"
        return
    }

# Configure the display of the cursor.
    Configure_Cursor .can working

# Set some defaults.
    set colour red; set lineWidth 4; set type ""

# Read the file into a list.
    set fileContents [Get_File_Contents $_MapFile]

    set defHtml 0
    set index 0

# While the index value is less than the list-length, loop through executing
# command.
    while {$index < [llength $fileContents]} {
        set line [lindex $fileContents $index]
        incr index
# If whitespace, goto next line.
        if {[regexp {^ *$} $line]} {
            continue
        }

# If this is a comment line, call proc that identifies comment values.
        if {[string range $line 0 0] == "#"} {
            Process_Comments $line colour lineWidth comments
        } else {

# Store the first 4 characters of current line in upper-case.
            set head [string toupper [string range $line 0 3]]

# Switch executes appropriate command.
            switch $head {
                RECT
                    -
                CIRC
                    -
                POLY {
                    set type [Extract_URL_Coords $line URL coords]

                    if {$lineWidth == 0} {
                        set style fill
                    } else {
                        set style outline
                    }
                    Create_Canvas_Object $type $coords $style $colour \
                        $lineWidth
                    set attrTags [list $type $colour $lineWidth $comments \
                        $URL]
                    .can.c1 addtag $attrTags withtag selected
                }
                DEFA {
                    set _Defaulthtml [string trim [string range \
                        [lindex $fileContents [expr $index-1]] 7 end]]
                    set defHtml 1
                }
                default {
                    postMessage "Uninterpretable/misplaced line in map file :\
                        \n $line"
                }
            }
        }
    }

# Once all canvas objects are created, check to see if the selected object has
# a URL attached.  Depending on the return value, set the state of the 
# "Test_URL" button.
    Configure_Test_URLbutton
# If there is no default html file, the imagemap mechanism will not work.
# Post a warning message to the user.
    if {$defHtml !=1} {
        postMessage "WARNING : Map file has no default html."
    }

# Invoke the menubutton to set default bindings, allowing all canvas objects
# to be selected and manipulated.
    .can.menu.create.men invoke @0

# Reset the cursor back to a red left_ptr cursor
    Configure_Cursor .can done
}

#=============================================================================#
# Gets the contents of a file. Each line stored as list element.
# args :
#       1. File to read.

proc Get_File_Contents {filename} {

    set file [open $filename r]

# If file could not be opened, post error message and return null-string.
    if {$file == ""} {
        postMessage "ERROR : opening file - $filename"
        return ""
    }

    set list ""

# Store each line of file as a list element
    while {1} {
        if {[eof $file]} {
            break
        }
        gets $file line
        lappend list $line
    }
    return $list
} 
#=============================================================================#
# Any comment line from the mapfile is passed in here.
# If this is the "attribute" comment line, appropriate vars are set.
# If it is just the comment for the url, it will be inserted in the 
# attributes tag.
# args :
#       1. Line read from mapfile.
#       2. Var to set objects colour to.
#       3. Var to set objects lineWidth to.
#       4. var to store comments in.

proc Process_Comments {line col lineW comm {autoLoad 1}} {

    global _ImageFile

    upvar 1 $col colour
    upvar 1 $lineW lineWidth
    upvar 1 $comm comments

# Split the comment line into the attribute part and the comment part.
    set listLine [split $line #]
    set line [lindex $listLine 1]
    set comments [lindex [lindex $listLine 2] 1]
# From the attributtes, extract relevant info.
    set words [lrange [split $line \ ] 1 4]

    set nwords 0

# While we are in the range of list elements, Identify the words in pairs.
    while {$nwords < [llength $words]} {
        set word1 [string range [string tolower [lindex $words $nwords]] 0 4]
        set word2 [lindex $words [expr ($nwords+1)]]
        incr nwords 2

# Switch executes appropriate command according to contents of var "word1".
        switch $word1 {
            _imag {
                if {$autoLoad} {
                    set _ImageFile $word2

                    if {$_ImageFile !=""} {
                        if {[yesNo "Load image file : $_ImageFile"] == "YES"} {
                            Image_Loader $_ImageFile
                        }
                    }
                }
            }
            _colo {
                set colour $word2
            }
            _line {
                set lineWidth $word2
            }
            "" {
                break
            }
        }
    }
}

#=============================================================================#
# Converts ambiguities between mapfile and tcl values for :
#     1. Object type.
#     2. Coordinate list format.
# args :
#       1. String from mapfile.
#       2. Var to hold URL.
#       3. Var to hold coordinate-list.

proc Extract_URL_Coords {line u c} {

    global _Absolute_URL

    upvar 1 $u URL
    upvar 1 $c coords

    set Type ""

# get the map line
    regsub -all "  " $line " " line
    set words [split $line \ ]

    if {[llength $words] < 3 && [lindex $words 0] != "default"} {
        postMessage "Bad (Map) line in file at: $line"
        return ""
    }
    
    set temp [string tolower [string range [lindex $words 0] 0 3]]

    switch $temp {
        rect {
            set Type rectangle
        }
        circ {
            set Type oval
        }
        poly {
            set Type polygon
        }
    }
    
    set URL [lindex $words 1]

# If there is currently no value for the absolute_URL path, then set it.
    if {$_Absolute_URL == ""} {
        set _Absolute_URL "[file dirname $URL]/"
    }

# extract the coords
    set End [llength $line]
    set coords0 [lrange $line 2 $End]
    regsub -all "," $coords0 " " coords

# If the object is a circle, amend the coordinates to hold values corresponding
# to tcl (top-left and bottom-right of a bounding box).
    if {$Type == "oval"} {
        set radius [expr ([lindex $coords 3]-[lindex $coords 1])]
        set x1 [expr ([lindex $coords 0]-$radius)]
        set x2 [expr ([lindex $coords 0]+$radius)]
        set y1 [expr ([lindex $coords 1]-$radius)]
        set y2 [expr ([lindex $coords 1]+$radius)]
        set coords "$x1 $y1 $x2 $y2"
    }
 
    return $Type
}

#=============================================================================#
proc Which_Save {} {

    global _Mode _NewImage

    Save_Map

    if {$_Mode == "maptoimage"} {
        Map_To_Image
#        Convert_To_Gif
        return
    }

    if {[info exists _NewImage]} {
        if {$_NewImage != ""} {
            set msg " The current Mode option is not correct for saving \
                images.  If you wish to save the image on each save command, \
                change the option under the Mode menu to \" Map -> Image \" \
                after this operation. \n\n\
                Do you wish to save the modified image."
 
            if {[yesNo $msg] == "YES"} {
                Map_To_Image
#                Convert_To_Gif
            } else {
                set _NewImage ""
            }
        }
    }
}

#=============================================================================#
proc Convert_To_Gif {} {

    global _NewImage env

    set root [file rootname $_NewImage]

    set msg "Do you want the modified image to be saved as a gif file ?"

    if {[yesNo $msg] == "YES"} {
        set msg "Do you want the image to be saved as : \n\ $root.gif"

        if {[yesNo $msg] == "NO"} {
            set return [Create_Browser]

            if {[lindex $return 1] != ""} {
                set gifFile "[lindex $return 0]/[lindex $return 1]" 

                while {[file exists $gifFile] && \
                    [yesNo "Overwrite existing file : $gifFile ?"]  == "NO"} {
                        set return [Create_Browser]
                        set gifFile "[lindex $return 0]/[lindex $return 1]"
                    
                }
            } else {
                postMessage " No filename was given. \n\
                    No image has been saved."
                return
            }
        }

        if {[info exists gifFile]} {
            set root1 [file rootname $gifFile]
        } else {
            set root1 $root
        }

        Configure_Message_Widget "\n Saving File : \n\ $root1.gif"
        Configure_Cursor .can working
        set qf [open /usr/tmp/quitfile$$ w]
        puts $qf "QUIT"
        exec $env(EDITIMAGE) $root.rppm $root.rgb < /usr/tmp/quitfile$$

        exec imconv $root.rgb $root.gif

        exec rm $root.rgb
        Configure_Cursor .can done
        Set_Message 1
    }
}
